/*********************************************************************
 *                                                                   *
 * Copyright (C) LANSA Group. All rights reserved                    *
 *                                                                   *
 *********************************************************************/

/**
* Spin Edit Dropdown extension definition.
*
* I wrapped the definition of this extension inside a function called DEFINE_EXTENSION_axSpinEdit.
* We don't have to put our extension definition inside a function, however it's a good idea
* to do that because our code declares variables (outside of a function variables are global - so we make
* them local by putting our code inside a function).
*
**/
function DEFINE_EXTENSION_axSpinEdit()
{
    /**
    * Call the function that defines the drop down item object
    **/
    defineSpinEditItemClass();

    /**
    * Define enumeration for the dropdown data source type
    **/
    dataSourceType =
    [
        {
            value: "xml",
            text:  TEXT.dev( "axDropDown.XMLfile" )
        },
        {
            value: "dynamic",
            text:  TEXT.dev( "axDropDown.DynamicTable" )
        },
        {
            value: "static",
            text:  TEXT.dev( "axDropDown.StaticTable" )
        },
        {
            value: "fixedValues",
            text:  TEXT.dev( "axDropDown.FixedValues" )
        },
        {
            value: "Range",
            text:  TEXT.cust( "axSpinEdit.Range" )
        }
    ];

    selectType =
    [
        {
            value: false,
            text:  TEXT.cust( "axSpinEdit.Single" )
        },
        {
            value: true,
            text:  TEXT.cust( "axSpinEdit.Multiple" )
        },
    ];

    /**
    * Drop down extension definition
    **/
    var definition_axSpinEdit =
    {
        /**
        * Programmatic class name of this extension. It's not visible to end user
        **/
        name: "axSpinEdit",

        group: "vis",

        /**
        * Type of object this extension is applicable. Can be either "element" or "screen"
        * If this extension applies to both element & screen, we put both in an array like so:
        * type: ["element", "screen"]
        **/
        type: "element",

        /**
        * End-user visible name of this extension
        **/
        display: TEXT.dev( "axSpinEdit.display" ),

        /* DO NOT REINITIALISE THIS VARIABLE */
        _tempInlineTable: "",

        /* State flag to know whether I have done the additional entries logic because it only needs to run once. */
        _additionalAdded: false,

        /**
        * End-user visible name of this extension
        **/
        properties:
        {
            dropDownStyle:
            {
                type: "Style"
            },

            /**
            * "dataSourceType" property indicates where the dropdown items come from.
            * The possible values are what we define earlier in dataSourceType
            **/
            dataSourceType:
            {
                type         : "String",
                items        : dataSourceType,
                defaultStatic: "fixedValues",
                description  : TEXT.dev( "axDropDown.dataSourceType.description" )
            },

            /**
            * "size" property indicates how many lines are visible
            * The possible values are what we define earlier in dataSourceType
            **/
            visibleLines:
            {
                type         : "String",
                defaultStatic: "2",
                description  : TEXT.cust( "axSpinEdit.visibleLines.description" )
            },

            /**
            * "multipleSelect" Single or multiple select.
            **/
            multipleSelect:
            {
                type         : "String",
                items        : selectType,
                defaultStatic: false,
                description  : TEXT.cust( "axSpinEdit.multipleSelect.description" )
            },

            /**
            * Lower limit value of Range
            *
            **/
            lowerRange:
            {
                type       : "String",
                description: TEXT.cust( "axSpinEdit.lowerRange.description" )
            },

            /** 
            * Upper limit value of Range
            *
            **/
            upperRange:
            {
                type       : "String",
                description: TEXT.cust( "axSpinEdit.upperRange.description" )
            },

            /**
            * Table name when the dataSourceType is static or dynamic
            *
            **/
            tableName:
            {
                type       : "String",
                description: TEXT.dev( "axDropDown.tableName.description" )
            },

            /**
            * "onFillDropDown" event
            * We raised this event to fill the drop down with data
            **/
            onFillDropDown:
            {
                type          : "Event",
                defaultDynamic: "ROW.text",
                description   : TEXT.dev( "axDropDown.onFillDropDown.description" )
            },

            /**
            * "xmlFileName" name of the xml file to load the table from.
            **/
            xmlFileName:
            {
                type       : "String",
                description: TEXT.dev( "axDropDown.xmlFileName.description" )
            },

            /**
            * "fileLocation" where to locate the file (xml or other(not currently implenented)).
            **/
            xmlFileLocation:
            {
                type         : "String",
                defaultStatic: "*DEFAULT",
                description  : TEXT.dev( "axDropDown.xmlFileLocation.description" )
            },

            /**
            * "fixedValues" values typed by the user when dataSourceType = "fixedValues".
            * We set the "isList" attribute to indicate a list.
            **/
            fixedValues:
            {
                type  : "axSpinEdititem",
                isList: true
            },

            /* allows users to add non table entries to a drop down - typically a blank entry  but can be anything, one or multiple entries. In the event the users must */
            /* call add */
            additionalEntries:
            {
                type       : "Event",
                description: TEXT.dev( "axDropDown.additionalEntries.description" )
            },

            /**
            * The drop down selected value
            **/
            onSelectValue:
            {
                type          : "Event",
                defaultDynamic: "ROW.value",
                description   : TEXT.dev( "axDropDown.onSelectValue.description" )
            },

            /**
            * "onSelectedValueChanged" event
            * We raised this event when the user change the selection in the drop down.
            **/
            onSelectedValueChanged:
            {
                type          : "Event",
                defaultDynamic: "FIELD.setValue( ROW.value )",
                description   : TEXT.dev( "axDropDown.onSelectedValueChanged.description" )
            },

            /* The name of the query specified in the dynamic table */
            sqlQueryName:
            {
                type:        "String",
                description: TEXT.dev( "axDropDown.sqlQueryName.description" )
            },

            /* Variables used in the query specified in the dynamic table */
            sqlVariables:
            {
                type:           "Event",
                defaultDynamic: "ENV.SQL.SQLVariableLibrary = USERENV.dftSQLDataLibrary",
                description:    TEXT.dev( "axDropDown.sqlVariables.description" )
            }
        },

        /**
        * This is where we put the code that initialise things (variable, HTML elements) that will remain
        * the same for the life of the extension.
        *
        * The "field" parameter is the abstraction of the field element and we reference this object
        * when we need to get or set the value of the field
        *
        * The "fieldDiv " parameter is the HTML element that serves as a container for the field.
        * Since it serves as a container for the field and not only for this extension, it might
        * already contain some HTML nodes already when this extension is executed.
        *
        * The reason why each extension does not get their own container is because some extensions
        * do not have their own visualisation (e.g. tooltip) and they only change the behaviour of
        * the associated field.
        *
        * For extensions like dropdown etc that have their own visualisation, we can assume
        * that the fieldDiv will be empty and we can ignore any elements there and create the 
        * elements we need. For the time being though when we are testing our extensions we need to
        * uncheck the "Default Visualisation" extension in order for our extension's visualisation to
        * appear (we might change this later to uncheck automatically not sure yet).
        **/
        init: function( field, fieldDiv )
        {
            var el = null;

            // use "isInput" property to test if the field is an input or output field
            if ( field.isInput )
            {
                // create a SELECT element for input field
                el            = document.createElement( "select" );
                el.size       = this.getPropertyValue( "visibleLines" );
                el.multiple   = this.getPropertyValue( "multipleSelect" );
                el.lowerRange = this.getPropertyValue( "lowerRange" );
                el.upperRange = this.getPropertyValue( "upperRange" );

                // hook up event handler for onchange event only if NOT in design mode
                if ( !this.isDesignMode() )
                {
                    var self = this;
                    el.onchange = function()
                    {
                        self._onChange( field, fieldDiv, this );
                    };
                }
            }
            else
                el = this.createTextElement();

            this.applyStyle( this.getPropertyValue( "dropDownStyle" ), el );

            /* Listen for changes in the field value */
            field.addListener( "ValueChanged", this._onFieldValueChanged, this );

            // save the reference to the element so we can use it later
            this._element = el;
            this.applyTabIndex( el );
            this._oFIELD = field;
            this._oDIV = fieldDiv;

            // append to the container
            fieldDiv.appendChild( el );
        },

        /**
        * We should put things that are impacted by change in properties here.
        *
        * The list of items is coming from the "fixedValues" property so the population
        * of the dropdown will happen in this method
        **/
        render: function( field, fieldDiv )
        {
            // el is the SELECT html element
            var el = this._element;
            var size = field.getSize();

            // set the size of the element to occupy the whole fieldDiv container
            if ( AXES.isTS2Engine )
            {
                el.style.width = size.width + "px";
                el.style.height = size.height + "px";
            }
            else
            {
                el.style.width = size.width;
                el.style.height = size.height;
            }

            // clear all existing item elements first
            var childNodes = el.childNodes;
            while ( childNodes.length > 0 )
                el.removeChild( childNodes[ 0 ] );

            var sSourceType = this.getPropertyValue( "dataSourceType" );
            var oTable = null;
            var inlineTable = this._tempInlineTable;
            var tableManager = AXES.TableManager;

            if ( this.isDesignMode() )
            {
                if ( "fixedValues" == sSourceType )
                {
                    oTable = tableManager.getTable( inlineTable );
                    oTable.removeChildren();
                }
                return;
            }

            var sTableName = "";
            switch ( sSourceType )
            {
            case "Range":

                var lowerRange = parseInt( this.getPropertyValue( "lowerRange" ) );
                var upperRange = parseInt( this.getPropertyValue( "upperRange" ) );
                oTable = tableManager.getTable( inlineTable );
                inlineTable = sTableName = oTable.name;

                if  ( oTable.childCount() <= 0 )
                {
                    if ( upperRange >= lowerRange )
                        for ( var i = lowerRange; i <= upperRange; i++ )
                            oTable.insertChild(
                                {
                                    value: 0 == i ? "" : i.toString(),
                                    text: i.toString()
                                }
                            );
                    else
                        /* allow the numbers to be added backwards */
                        for ( var i = lowerRange; i >= upperRange; i-- )
                            oTable.insertChild(
                                {
                                    value: 0 == i ? "" : i.toString(),
                                    text: i.toString()
                                }
                            );
                }
                break;

            case "fixedValues":

                oTable = tableManager.getTable( inlineTable );
                inlineTable = sTableName = oTable.name;

                if ( oTable.childCount() <= 0 )
                {
                    var item;
                    var items = this.getPropertyValue( "fixedValues" );
                    for ( var i in items )
                    {
                        item = items[ i ];
                        oTable.insertChild(
                            {
                                value: item.value,
                                text: item.text
                            }
                        );
                    }
                }
                break;

            case "static":

                sTableName = this.getPropertyValue( "tableName" );
                oTable = tableManager.getTable( sTableName );
                tableManager.loadStaticTables( USERENV.staticTablesFile, {}, false );
                break;

            case "dynamic":

                var env =
                {
                    SQL: {}
                };
                this.raiseEvent( "sqlVariables", env );

                sTableName = this.getPropertyValue( "sqlQueryName" );
                tableManager.loadDynamicTable( sTableName, USERENV.dynamicTablesFile, env.SQL );
                oTable = tableManager.getTable( sTableName );
                break;

            case "xml":

                var sPath = this.getPropertyValue( "xmlFileLocation" );
                if ( sPath == "*DEFAULT" )
                    sPath = tableManager.getDefaultPath();

                sTableName = sPath + "/" + this.getPropertyValue( "xmlFileName" );
                oTable = tableManager.getTable( sTableName );
                tableManager.loadFromXML( sTableName, oTable, false );

                break;
            }

            var env;
            var bSelected = false;
            var sFldValue = TRIM( field.getValue() );

            /* store away the table name for the addTableROW method to use */
            this.TableName = sTableName;

            if ( field.isInput )
            {
                if ( !this._additionalAdded )
                {
                    this.raiseEvent( "additionalEntries" );
                    this._additionalAdded = true;
                }

                var groupLabel = null;
                var optParent = null;
                var useOptGroup = false;
                var sDropDownText, oOption, oRow;
                var oSelectedArray = sFldValue.split( ',' );

                for ( var i = 0; i < oTable.childCount(); i++ )
                {
                    oRow = oTable.child( i );
                    env =
                    {
                        ROW  : oRow,
                        FIELD: field
                    };
                    sDropDownText = this.getPropertyValue( "onFillDropDown", env );

                    if ( sDropDownText )
                    {
                        oOption = document.createElement( "option" );
                        $( oOption ).text( sDropDownText );
                        oOption._RowRef = oRow;

                        if ( useOptGroup )
                            optParent.appendChild( oOption );
                        else
                            el.appendChild( oOption );

                        oOption.selected = false;

                        if ( !bSelected || el.multiple ) /* only keep comparing when a value has already been selected if have multiple selections*/
                        {
                            bSelected = this.getPropertyValue( "onSelectValue", env );
                            if ( typeof bSelected != "boolean" )
                            {
                                bSelected = false;
                                for ( var j in oSelectedArray )
                                    if ( this.getPropertyValue( "onSelectValue", env ) == oSelectedArray[ j ] )
                                    {
                                        // if a match is found then set to selected
                                        bSelected = true;
                                        break;
                                    }
                            }
                            oOption.selected = bSelected;
                        }
                    }
                    else
                    {
                        groupLabel = oRow.optgroup;
                        if ( groupLabel )
                        {
                            var oOptGroup = document.createElement( "optgroup" );
                            oOptGroup.label = groupLabel;
                            el.appendChild( oOptGroup );
                            optParent = oOptGroup;
                            useOptGroup = true;
                        }
                    }
                } /* end for */
            }  /* end if input */
            else /* it's an output field */
            {
                for ( var i=0; i < oTable.childCount(); i++ )
                {
                    env =
                    {
                        ROW  : oTable.child( i ),
                        FIELD: field
                    };

                    bSelected = this.getPropertyValue( "onSelectValue", env );

                    if ( typeof bSelected != "boolean" )
                        bSelected = ( this.getPropertyValue( "onSelectValue", env ) == sFldValue );

                    if ( bSelected )
                    {
                        el.innerText = this.getPropertyValue( "onFillDropDown", env );
                        break;
                    }
                } /* end for */
            } /* end if input */
        },

        addTableROW: function ( oRow, bAtBegin )
        {
            AXES.TableManager.getTable( this.TableName ).insertChild( oRow, bAtBegin );;
        },

        /**
        * Do opposite of init
        **/
        destroy: function( field, fieldDiv )
        {
            if ( AXES.isTS2Engine && "static" == this.getPropertyValue( "dataSourceType" ) )
                AXES.TableManager.unloadStaticTables();

            // remove the element referenced by this._element from the DOM
            var el = this._element;
            if ( el )
            {
                el.onchange = el.size = el.multiple = null;
                el.parentNode.removeChild( el );
                delete el;
            }

            field._oFIELD = field._oDIV = null;

            /* Remove any listeners */
            field.removeListener( "ValueChanged", this._onFieldValueChanged );
        },

        /**
        * Handles the onchange event of the dropdown
        **/
        _onChange: function( field, fieldDiv )
        {
            if ( this.isDesignMode() )
            {
                return;
            }

            this._noFieldValueChangedEvent = true;

            //
            var el = this._element;
            var options = el.options;
            var oSelectedOption;

            if ( el.multiple )
            {
                var oSelectedArray = [];
                for ( var i in options )
                    if ( !isNaN( parseInt( i ) ) && options[ i ].selected )
                        oSelectedArray.push( options[ i ]._RowRef );

                // returns array if multiple select
                oSelectedOption = oSelectedArray;
            }
            else oSelectedOption = options[ options.selectedIndex ]._RowRef;
            this.raiseEvent("onSelectedValueChanged", { ROW : oSelectedOption, FIELD: field } );
        },

        /* Handles the change in field value */
        _onFieldValueChanged: function( event )
        {
            var field = this._oFIELD;
            if ( !field.isInput )
            {
                return;
            }

            if ( this._noFieldValueChangedEvent )
            {
                delete this._noFieldValueChangedEvent;
                return;
            }

            var env, bSelected;
            var oDD = this._element;
            var options = oDD.options;
            var oSelectedArray = [];
            oSelectedArray = TRIM( field.getValue() ).split(',');
            var getPropertyValue = this.getPropertyValue;

            for ( var i in options )
            {
                // set every option to not selected
                options[ i ].selected = false;
                env =
                {
                    ROW  : options[ i ]._RowRef,
                    FIELD: field
                };
                bSelected = getPropertyValue( "onSelectValue", env );
                if ( typeof bSelected != "boolean" )
                {
                    bSelected = false;
                    for ( var j in oSelectedArray )
                    {
                        if ( getPropertyValue( "onSelectValue", env ) == oSelectedArray[ j ] )
                        {
                            // if a match is found then set to selected
                            bSelected = true;
                            break;
                        }
                    }
                }
                options[ i ].selected = bSelected;
                if ( bSelected  && !oDD.multiple )
                    break;
            }
        }
    };

    /**
    * Register the extension with AXES
    **/
    AXES.Extensions.add( definition_axSpinEdit );

    /**
    * The function belows defines the drop down item object.
    * Each drop down item has a "value" and "text" property.
    * The "value" property is what will be used as the actual 5250 field value and
    * the "text" property is what will be displayed on the drop down.
    **/
    function defineSpinEditItemClass()
    {
        /** 
        * constructor function for the drop down item object
        **/
        var itemClass = function()
        {
            this.text = this.value = "";
        };

        /**
        * "toString" function for the item object.
        * This will be used by the designer when it needs to display a simple representation
        * of the drop down item.
        **/
        itemClass.prototype.toString = function()
        {
            var value = this.getPropertyValue( "value" );
            var text = this.getPropertyValue( "text" );
            return value + " - " + text;
        };

        /**
        * Properties of drop down item object.
        **/
        var itemClassDef =
        {
            value:
            {
                type: "String"
            },
            text:
            {
                type: "String"
            }
        };

        /**
        * We need to register the constructor function and property list with AXES.
        * We want to name the item object class "axSpinEdititem".
        **/
        AXES.Extensions.definePropertyType(
            "axSpinEdititem",
            {
                objectClass     : itemClass,
                objectDefinition: itemClassDef
            }
        );
    }
}

/**
* Make sure we call the function we just defined.
**/
DEFINE_EXTENSION_axSpinEdit();

